﻿using AutoMapper;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class FileService : IFileService
    {
        private readonly Tsi1Context _context;

        private readonly IMapper _mapper;

        private readonly string _path;

        public FileService(Tsi1Context context, IMapper mapper, IHostingEnvironment hostingEnvironment)
        {
            _context = context;
            _mapper = mapper;
            _path = hostingEnvironment.ContentRootPath + "/StaticFiles";
        }

        public async Task<ServiceResult<string>> SaveFile(IFormFile file, string filePath)
        {
            var result = new ServiceResult<string>();

            var fileStream = new FileStream(filePath, FileMode.Create);

            try
            {
                await file.CopyToAsync(fileStream);
            }
            catch (Exception)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.ErrorSavingFile, filePath);
                return result;
            }
            finally
            {
                fileStream.Close();
            }

            return result;
        }

       

        public async Task<ServiceResult<FileDto>> Create(IFormFile file, int tenantId, int courseId)
        {
            var result = new ServiceResult<FileDto>();

            var tenant = await _context.Tenants
                .FirstOrDefaultAsync(x => x.Id == tenantId);

            var course = await _context.Courses
                .FirstOrDefaultAsync(x => x.Id == courseId);

            var filePath = Path.Combine(_path, tenant.Name, course.Name, file.FileName);

            var existingFile = await _context.Files.FirstOrDefaultAsync(x => x.Path == filePath);
            if (existingFile != null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.DuplicateFile, filePath);
                return result;
            }

            var newFile = new DataLayer.Entities.File
            {
                Name = file.FileName,
                Path = filePath.Replace(_path, ""),
            };

            _context.Add(newFile);
            await _context.SaveChangesAsync();

            var resultSaveFile = await this.SaveFile(file, filePath);
            if (resultSaveFile.HasError)
            {
                result.HasError = true;
                result.Message = resultSaveFile.Message;

                _context.Remove(newFile);
                await _context.SaveChangesAsync();

                return result;
            }

            var fileDto = _mapper.Map<FileDto>(newFile);
            result.Data = fileDto;

            return result;
        }


        public async Task<ServiceResult<bool>> DeleteFile(string filePath)
        {
            var result = new ServiceResult<bool>();
            
            try
            {
                using (var fileStream = new FileStream(filePath, FileMode.Open, FileAccess.Read, FileShare.None, 4096, FileOptions.DeleteOnClose)) {}
            }
            catch (Exception)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.ErrorDeletingFile, filePath);
                return result;
            }

            return result;
        }
    }
}
